diff --git a/src/Fl.cxx b/src/Fl.cxx
index 9e7aa0d..8d68d5d 100644
--- a/src/Fl.cxx
+++ b/src/Fl.cxx
@@ -50,6 +50,8 @@
 #include <FL/Fl_Tooltip.H>
 #include <FL/x.H>
 
+#include <FL/names.h>
+
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -1150,7 +1152,24 @@ static int send(int event, Fl_Widget* to, Fl_Window* window) {
     if (w->type()>=FL_WINDOW) {dx -= w->x(); dy -= w->y();}
   int save_x = Fl::e_x; Fl::e_x += dx;
   int save_y = Fl::e_y; Fl::e_y += dy;
+
+  if (event == FL_KEYBOARD || event == FL_SHORTCUT) {
+    printf("send(event=%2d (%-11s), text='%s', to=%p (wi%s) %10s %-5s %-10s '%-8.8s', window=%-9p",
+      event, fl_eventnames[event], Fl::event_text(),
+      to,to->as_window() ? "ndow" : "dget",
+      to->active_r() ? "active_r" : "inactive_r",
+      (Fl::focus()==to) ? "focus" : "",
+      (Fl::belowmouse()==to) ? "belowmouse" : "",
+      to ? to->label() : "",
+      window);
+    fflush(stdout);
+    dx += 1;	// for debugging only (to set breakpoint)
+  }
   int ret = to->handle(Fl::e_number = event);
+  if (event == FL_KEYBOARD || event == FL_SHORTCUT) {
+    printf(") = %d\n",ret);
+    dx -= 1;	// for debugging only (to set breakpoint)
+  }
   Fl::e_number = old_event;
   Fl::e_y = save_y;
   Fl::e_x = save_x;
diff --git a/test/input.cxx b/test/input.cxx
index 33f8abc..187e900 100644
--- a/test/input.cxx
+++ b/test/input.cxx
@@ -87,6 +87,12 @@ int main(int argc, char **argv) {
   Fl::args(argc, argv);
   Fl::get_system_colors();
   Fl_Window *window = new Fl_Window(400,420);
+  Fl_Group *grp = new Fl_Group(0,0,400,420,"Group");
+
+  printf("\n");
+  printf("--------------------\n");
+  printf("--- test started ---\n");
+  printf("--------------------\n\n");
 
   int y = 10;
   input[0] = new Fl_Input(70,y,300,30,"Normal:"); y += 35;
@@ -106,8 +112,11 @@ int main(int argc, char **argv) {
   input[4]->tooltip("Input field for short text with newlines (Alt-M)");
   input[4]->wrap(1);
 
-  for (int i = 0; i < 4; i++) {
+  printf(" Group    = %p, '%s'\n", grp, grp->label());
+
+  for (int i = 0; i < 5; i++) {
     input[i]->when(0); input[i]->callback(cb);
+    printf(" input[%d] = %p, '%s'\n", i, input[i], input[i]->label());
   }
   int y1 = y;
 
@@ -129,11 +138,11 @@ int main(int argc, char **argv) {
   b->callback(button_cb);
   b->tooltip("Print widgets that have changed() flag set");
 
-  b = new Fl_Light_Button(10,y,100,25," Tab Nav");
+  b = new Fl_Light_Button(10,y,100,25," &Tab Nav");
   b->tooltip("Control tab navigation for the multiline input field");
   b->callback(tabnav_cb, (void*)input[4]);
   b->value(input[4]->tab_nav() ? 1 : 0);
-  b = new Fl_Light_Button(110,y,100,25," Arrow Nav"); y += 25;
+  b = new Fl_Light_Button(110,y,100,25," A&rrow Nav"); y += 25;
   b->tooltip("Control horizontal arrow key focus navigation behavior.\n"
              "e.g. Fl::OPTION_ARROW_FOCUS");
   b->callback(arrownav_cb);
@@ -143,7 +152,7 @@ int main(int argc, char **argv) {
   b = new Fl_Button(220,y1,120,25,"color"); y1 += 25;
   b->color(input[0]->color()); b->callback(color_cb, (void*)0);
   b->tooltip("Color behind the text");
-  b = new Fl_Button(220,y1,120,25,"selection_color"); y1 += 25;
+  b = new Fl_Button(220,y1,120,25,"selection_&color"); y1 += 25;
   b->color(input[0]->selection_color()); b->callback(color_cb, (void*)1);
   b->labelcolor(fl_contrast(FL_BLACK,b->color()));
   b->tooltip("Color behind selected text");
@@ -151,9 +160,13 @@ int main(int argc, char **argv) {
   b->color(input[0]->textcolor()); b->callback(color_cb, (void*)2);
   b->labelcolor(fl_contrast(FL_BLACK,b->color()));
   b->tooltip("Color of the text");
-
+  grp->end();
   window->end();
   window->show(argc,argv);
+
+  grp->deactivate();
+  input[3]->deactivate(); // Secret
+
   return Fl::run();
 }
 
